@@ -67,3 +67,10 @@ to reside. This is set automatically from sysconfdir by ./configure .*/
 
 /*Define if running on digital.*/
 #undef BROKEN_TERMIOS_VMIN
+
+
+/*Controls whether or not QUEUE_MANAGER package is compiled in.*/
+#undef NO_QUEUE_MANAGER
+
+
+
