@@ -1,3 +1,6 @@
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #ifdef HAVE_ARPA_H
 #include <arpa/inet.h>
 #endif
@@ -150,9 +153,6 @@
 #ifdef HAVE_UTMP_H
 #include <utmp.h>
 #endif
-#ifdef HAVE_VARARGS_H
-#include <varargs.h>
-#endif
 #ifdef HAVE_VFORK_H
 #include <vfork.h>
 #endif
