@@ -1,6 +1,6 @@
 /*
  *  Queue load balancing system
- *  $Revision: 1.6 $
+ *  $Revision: 1.8 $
  *
  *  Copyright (C) 1998-2000 W. G. Krebs
  *
@@ -40,7 +40,7 @@ extern int check_ident();
 
 extern void mkutmp();
 
-int fd3, fd22, collected; 
+int fd3, fd22, collected;
 char mode;
 
 #ifdef __hpux
@@ -52,11 +52,11 @@ struct sigaction mysignal;
 
 int controlmsg (char *string);
 
-void 
-do_nothing(void) 
+void
+do_nothing(void)
 {
   /*This is a hack, obviously.*/
-} 
+}
 
 char *fgetl(FILE *stream);
 
@@ -76,7 +76,7 @@ int checkpointmode = 0;
 int restartmode = 0;
 char *restartfile = NULL;
 
-int compar(const void *one, const void *two) 
+int compar(const void *one, const void *two)
 {
   return(strcmp((char*)one, (char*) two));
 }
@@ -91,7 +91,7 @@ int loadstop = 0;
 
 char ttyinput, ttyoutput, ttyerror;
 
-void 
+void
 chldsigh(void)
 {
   char val;
@@ -103,9 +103,9 @@ chldsigh(void)
     are probably other ways for a determined user to tie up the queue,
     so what's the point.*/
   /*    if((donthang==1)&&(WIFSTOPPED(newwait)!=0)) {
-      fprintf(queue, 
+      fprintf(queue,
 	      "\nShell was killed by queued because\n it was stopped \
-	      in queue by signal %d.\n", 
+	      in queue by signal %d.\n",
 	      WSTOPSIG(newwait));
       kill(-pid, SIGTERM);
       sleep(1);
@@ -126,14 +126,14 @@ chldsigh(void)
     control.  This way stub does not suspend under these
     circumstances.*/
   if ((donthang==0)&&((loadstop==0)
-		      ||!(WIFSTOPPED(newwait)))) 
+		      ||!(WIFSTOPPED(newwait))))
     send(fdsock2, &val, sizeof(val), MSG_OOB);
 
   /*Make sure we terminate*/
   if(WIFSTOPPED(newwait)==0) dead = 1;
 }
 
-void 
+void
 bigsigh(int n)
 {
   /* 95/7/7 WK: The purpose of this hack is to propagate signals from
@@ -168,7 +168,7 @@ bigsigh(int n)
 	to return SIGCHKPT as the cause of death.*/
 
       sigsetmask(~0L);
-      ret = checkpoint(pid, fd, 
+      ret = checkpoint(pid, fd,
 		       CKPT_CHECK_ALL|CKPT_KILL|CKPT_NO_SHARED_LIBRARIES
 		       |CKPT_NO_BINARY_FILE);
       /*Process should die now. Report the fact that it was
@@ -226,7 +226,7 @@ bigsigh(int n)
     mysignal.sa_handler = SIG_IGN;
 
     /*This is the most portable version, it assumes
-      that the signal mask is at the start of the mysignal object 
+      that the signal mask is at the start of the mysignal object
       (which is a structure on some systems and a long int on others),
       where it normally is.*/
 
@@ -242,7 +242,7 @@ bigsigh(int n)
       the other 64 signals that are partially defined in <sigset.h>*/
     mysignal.sa_flags = SA_RESTART;
     sigaction(n, &mysignal, NULL);
-#endif 
+#endif
     /*Allow pending signal to come in and be ignored.*/
     sigsetmask(~sigmask(n));
     /*Restore signal mask Asap.*/
@@ -260,7 +260,7 @@ bigsigh(int n)
     int nine = SIGKILL;
     /*Give process group some time to think it over and say last prayers.*/
 
-    sleep(1); 
+    sleep(1);
 
     /*WGK 1998/08/05 It might be desirable to increase
       this time or provide the queue user an option to increase this
@@ -281,7 +281,7 @@ bigsigh(int n)
   }
 }
 
-int 
+int
 controlmsg(char *string)
 {
   int desc;
@@ -309,7 +309,7 @@ controlmsg(char *string)
 extern int globalargc;
 extern char **globalargv;
 
-void 
+void
 handle(char *thefile, int checkpoint, int restart, char *restartdata)
 {
   static int num = 0;
@@ -345,7 +345,7 @@ handle(char *thefile, int checkpoint, int restart, char *restartdata)
   int pty1, pty2;
 
   char *userid, *mailuserid, *jobname, *onlyhost, *cookie;
-  
+
   restartfile = restartdata;
   restartmode = restart;
   checkpointmode = checkpoint;
@@ -368,7 +368,7 @@ handle(char *thefile, int checkpoint, int restart, char *restartdata)
   while (again==1) {
     again=0;
     mode = 'i';
-    
+
     sigsetmask(~0L);
 
     mail = 1;
@@ -394,12 +394,12 @@ handle(char *thefile, int checkpoint, int restart, char *restartdata)
     /*The following three pieces of info are needed at the beginning
      because jobinfo() reads them in first for logging info.*/
 
-    fread(&uid, sizeof(int), 1, queue);
+    netfread(&uid, sizeof(int), 1, queue);
     userid=fgetl(queue);
     mailuserid=fgetl(queue);
     jobname=fgetl(queue);
     onlyhost=fgetl(queue); /*Batchd already checked this for us.*/
-    fread(&donthang, sizeof(donthang), 1, queue);
+    netfread(&donthang, sizeof(donthang), 1, queue);
 
     cookie=fgetl(queue); /*it general, it prevents a mix-up of ports
 			   if queue.c has been killed in the mean-time
@@ -407,15 +407,16 @@ handle(char *thefile, int checkpoint, int restart, char *restartdata)
 			   port.  With NO_ROOT, prevents malicious
 			   users; in general, it prevents*/
 
+    /* EJD - 8/14/2000 - Don't need to swap host address */
     fread(&(myclient.sin_addr), sizeof(myclient.sin_addr), 1, queue);
     {int tmpport;
-    fread(&tmpport, sizeof(int), 1, queue);
+    netfread(&tmpport, sizeof(int), 1, queue);
     rport = htons((u_short) tmpport);
-    } 
+    }
 
    {register int i;
    int adj = 0;
-   fread(&args, sizeof(int), 1, queue);
+   netfread(&args, sizeof(int), 1, queue);
 
 #ifdef ENABLE_CHECKPOINT
 
@@ -439,24 +440,24 @@ handle(char *thefile, int checkpoint, int restart, char *restartdata)
    pipe(filedes);
 
    dirname=fgetl(queue);
-   fread(&aid, sizeof(int), 1, queue);
-   fread(&euid, sizeof(int), 1, queue);
-   fread(&egid, sizeof(int), 1, queue);
-   fread(&gid, sizeof(int), 1, queue);
-   fread(&ngroups, sizeof(int), 1, queue);
+   netfread(&aid, sizeof(int), 1, queue);
+   netfread(&euid, sizeof(int), 1, queue);
+   netfread(&egid, sizeof(int), 1, queue);
+   netfread(&gid, sizeof(int), 1, queue);
+   netfread(&ngroups, sizeof(int), 1, queue);
 
    if((gidset = malloc(ngroups*sizeof(gid_t)))==0) {
      syslog(LOG_ERR, "Out of memory.\n");
      exit(2);
    }
 
-   fread(gidset, sizeof(gid_t), ngroups, queue);
-   fread(&ttyinput, sizeof(ttyinput), 1, queue);
-   fread(&ttyoutput, sizeof(ttyoutput), 1, queue);
-   fread(&ttyerror, sizeof(ttyoutput), 1, queue);
-   fread(&globalterm, sizeof(globalterm), 1, queue);
+   netfread(gidset, sizeof(gid_t), ngroups, queue);
+   netfread(&ttyinput, sizeof(ttyinput), 1, queue);
+   netfread(&ttyoutput, sizeof(ttyoutput), 1, queue);
+   netfread(&ttyerror, sizeof(ttyoutput), 1, queue);
+   netfread(&globalterm, sizeof(globalterm), 1, queue);
 #ifdef TIOCSWINSZ
-   fread(&ws, sizeof(ws), 1, queue);
+   netfread(&ws, sizeof(ws), 1, queue);
 #endif
    num = 0;
    while (*(estack[num] = fgetl(queue))!=0) {
@@ -469,9 +470,9 @@ handle(char *thefile, int checkpoint, int restart, char *restartdata)
    sprintf(tmp, "QUEUE=%c", mode);
    estack[num]=tmp;
    estack[num+1]=0;
-   fread(&mask, sizeof(mode_t), 1, queue);
-   fread(&prio, sizeof(int), 1, queue);
-   fread(myrlimit, sizeof(struct rlimit), 8, queue);
+   netfread(&mask, sizeof(mode_t), 1, queue);
+   netfread(&prio, sizeof(int), 1, queue);
+   netfread(myrlimit, sizeof(struct rlimit), 8, queue);
    /*unlink(file);*/
    /*fclose(queue);*/
    if(donthang==0) {
@@ -538,7 +539,7 @@ handle(char *thefile, int checkpoint, int restart, char *restartdata)
 
 	 if (errno == ECONNREFUSED & slptim <= 4) {
 	   /*Connection refused; maybe server just overloaded? Sleep a bit, then try again.*/
-	    
+	
 	   sleep(slptim);
 	   slptim += 2;	
 	   continue;
@@ -547,8 +548,8 @@ handle(char *thefile, int checkpoint, int restart, char *restartdata)
 	 if (hostp->h_addr_list[1] != NULL) {
 	   /*Be sure to try all possible addrs for the host.*/
 	   perror((char *) 0);
-	    
-	   hostp->h_addr_list++;    
+	
+	   hostp->h_addr_list++;
 	   bcopy(hostp->h_addr_list[0], (caddr_t) &address1.sin_addr,
 		 hostp->h_length);
 
@@ -563,12 +564,12 @@ handle(char *thefile, int checkpoint, int restart, char *restartdata)
 		    }*/
 	   continue;
 	 }
-	  
+	
 	 /*perror(hostp->h_name);*/	/*so long*/
 	 sigsetmask(oldmask);
 	 goto berror;
        }
-      
+
        {
 	 /*We create a second socket which handles sterr and also
 	   control messages between the clients. This needs to be on a
@@ -635,9 +636,9 @@ crypt().*/
 
       /*Send cnounce, as compliant with draft protocol. WGK 2000/07/20*/
       write(fdsock1, &cnounce, strlen(cnounce)+1);
- 
+
       /*Read nounce, as compliant with draft protocol. WGK 2000/07/20*/
- 
+
       alarm(5);
       k = 0;
       do {
@@ -674,7 +675,7 @@ crypt().*/
     /*OK, cnounce read, now generate nounce.*/
     randomstr(&nounce[0]);
     write(fdsock1, &nounce, strlen(nounce)+1);
-    
+
     strcpy(bigcookie,nounce);
     strcat(bigcookie,cookie);
     strcat(bigcookie,cnounce);
@@ -710,7 +711,7 @@ crypt().*/
 
 
     }
-    
+
     }
 
 	 listen(tempsocket, 1);
@@ -876,7 +877,7 @@ crypt().*/
      */
 
      mysignal.sa_flags = SA_RESTART; /*BSD signal behavior*/
-     sigaction(SIGCHLD, &mysignal, NULL);    
+     sigaction(SIGCHLD, &mysignal, NULL);
      mysignal.sa_handler = bigsigh;
      sigaction(SIGUSR1, &mysignal, NULL);
      sigaction(SIGUSR2, &mysignal, NULL);
@@ -992,15 +993,23 @@ crypt().*/
 #endif /*NO_ROOT*/
 
      setpriority(PRIO_PROCESS, 0, prio);
-     for(i=0;i<8;++i) {
-       static struct rlimit templimit;
+     /*   for(i=0;i<8;++i) {
+       static struct rlimit templimit;*/
        /*Do not
+
+WGK 2000/08/11 Eric Deal <eric.deal@conexant.com> pointed out that this
+"Do not" comment looks bogus, because it goes too far into the code.
+
+The entire for loop should be commented out, which I've now this. This
+probably created another bug involving Xwindows applications that close
+stdin and stdout that wasn't present in old versions of Queue.
+*//*
 	 getrlimit(i, &templimit);
 
 	 (templimit.rlim_curr < myrlimit[i].rlim_curr ? myrlimit[i].rlim_curr = templimit.rlim_curr)
 	 (templimit.rlim_max < myrlimit[i].rlim_max ? myrlimit[i].rlim_max = templimit.rlim_max)
 	 ssetrlimit(i, &myrlimit[i]);
-	 }
+	 }*/
 
 	 closelog();
 
@@ -1132,13 +1141,13 @@ crypt().*/
 	 }
 	 {
 	   register int i;
-#ifdef __hpux 
-	   for(i=0;i<64;++i) sigvector(i, SIG_DFL, NULL);    
+#ifdef __hpux
+	   for(i=0;i<64;++i) sigvector(i, SIG_DFL, NULL);
 #else
 	   /*GNU/Linux*/
 	   struct sigaction myaction;
 	   myaction.sa_handler= SIG_DFL;
-	   for(i=0;i<64;++i) sigaction(i, &myaction, NULL);    
+	   for(i=0;i<64;++i) sigaction(i, &myaction, NULL);
 #endif
 	   sigsetmask(0L);
 	 }
@@ -1149,18 +1158,18 @@ crypt().*/
 	   if (ret!=0)
 	     {
 	       fprintf(stderr, "Error (%d) preparing to collect data\n", ret);
-	       exit(2); 
+	       exit(2);
 	     }
 	 }
 
 	 if((checkpoint==KERNEL_CHECKPOINT) && flag && restart == RESTART && restartdata && *restartdata) {
 	   mrestart(restartdata);
 	   /*Normally not reached.*/
-	   fprintf(stderr, "QueueD: handle: In process migration, unable to restart file %s\n", 
+	   fprintf(stderr, "QueueD: handle: In process migration, unable to restart file %s\n",
 		   restartdata);
 	   exit(2);
 	 }
-#endif /*HAVE_ASM_CHECKPOINT_H*/     
+#endif /*HAVE_ASM_CHECKPOINT_H*/
 
 	 if (flag) execve(file, command, estack);
        }
@@ -1216,14 +1225,14 @@ crypt().*/
 		 rest.*/
 	       FD_SET(fdpipe[0], &ready);
 	       goto drain;}
-	   }  
+	   }
 	 }
 	 continue;
 	 }
      drain:				
        if (FD_ISSET(fdsock2, &ready)) {
 	 static char sigval;
-	 if ((read(fdsock2, &sigval, 1)) <=0) { 
+	 if ((read(fdsock2, &sigval, 1)) <=0) {
 	   /*The death of the control stream
 	     is ominous. We invoke the
 	     our SIGTERM handler.*/
@@ -1249,7 +1258,7 @@ crypt().*/
 	      but it gets complicated and may prevent the user from
 	      killing stopped jobs before they restart.*/
 
-	   if(((int)sigval==SIGCONT)&(ttyinput||ttyoutput)) 
+	   if(((int)sigval==SIGCONT)&(ttyinput||ttyoutput))
 	     tcsetpgrp(pty1, pid);
 
 
@@ -1316,7 +1325,7 @@ crypt().*/
 	     goto drain;}
 	 }
        }
-			      
+			
      } while (FD_ISSET(fdsock2, &readfrom) ||
 	      FD_ISSET(fdpipe[0], &readfrom)||
 	      ((ttyinput|ttyoutput)&&FD_ISSET(pty1, &readfrom))
@@ -1363,7 +1372,7 @@ crypt().*/
    /*Simulate exit status of child.*/
 
    if(WIFEXITED(mywait)!=0) exit(WEXITSTATUS(mywait));
-   if(WCOREDUMP(mywait)==0) { 
+   if(WCOREDUMP(mywait)==0) {
      /*Process did not dump core, so prevent a simulated core dump.*/
      struct rlimit rlp;
      rlp.rlim_cur = 0;
@@ -1375,7 +1384,7 @@ crypt().*/
 
      setrlimit(RLIMIT_CORE, &rlp);
    }
-   if(WIFSIGNALED(mywait)==0) 
+   if(WIFSIGNALED(mywait)==0)
      fprintf(queue, "\nShell terminated from unknown cause.\n");
    fflush(stdout);
    fclose(queue);
