@@ -3,7 +3,8 @@ enum keyword {
 	K_CONSOLE, K_NICE, K_RESTART, K_TIMECMD, K_DAY,
 	K_OFF, K_DRAIN, K_ON, K_NUMBER, K_VARIABLE, K_LINE,
 	K_TIMESTOP, K_TIMESCHED,
-	K_SYNTAX, K_LOADSTOP, K_LOADSCHED,
+	K_SYNTAX, K_LOADSTOP, K_LOADSCHED,K_LOADCHECKPOINT,K_RESTARTMODE,
+        K_RESTARTFLAG, K_CHECKPOINTMODE,
 	K_MINFREE,
 	K_PROGRAM,
 	K_RLIMITCPU,
