@@ -1,6 +1,6 @@
 /*
  *  Queue load balancing system
- *  $Revision: 1.2 $
+ *  $Revision: 1.3 $
  *  Copyright (C) 1998-2000 W. G. Krebs <wkrebs@gnu.org>
  *
  *  wkrebs@gnu.org
@@ -142,10 +142,18 @@ mkutmp(char *name, int pty2, int uid, int gid)
   char buf[255];
   line = mtos();
   strcpy(myu.ut_user, name);
-  strcpy(myu.ut_id, line+12);
+#ifdef linux
+  strncpy(myu.ut_id,line+8,2);
+#else
+  strncpy(myu.ut_id, line+12,2);
+#endif
   strcpy(buf, "/dev/tty");
   strcat(buf, myu.ut_id);
+#ifdef linux
+  strcpy(myu.ut_line, line+5);
+#else
   strcpy(myu.ut_line, line+9);
+#endif
   myu.ut_pid = getpid();
   myu.ut_type = USER_PROCESS;
   myu.ut_time = time(NULL);
@@ -158,8 +166,14 @@ mkutmp(char *name, int pty2, int uid, int gid)
   pututline(&myu);
   fchown(pty2, uid, gid);
   fchmod(pty2, S_IRUSR|S_IWUSR);
+
+  if(!strcmp(buf,"/dev/tty")) {
+    syslog(LOG_ERR,"queue: pty.c: mkutmp: bug: buf equals /dev/tty.");
+  }
+  else {
   chown(buf, uid, gid);
   chmod(buf, S_IRUSR|S_IWUSR);
+  }
 }
 
 /*HP ptsname is buggy! So we write our own!*/
