@@ -1,6 +1,6 @@
 /*
  *  Queue load balancing system
- *  $Revision: 1.5 $
+ *  $Revision: 1.6 $
  *
  *  Copyright (C) 1998-2000 W. G. Krebs <wkrebs@gnu.org>
  *  ReadHosts() Copyright (C) 1998 Free Software Foundation, Inc.
@@ -88,11 +88,11 @@ ReadHosts(char *HostFile)
     free up the structure.*/
 
   if(Hosts) {
-    for(i = 0;i < NHosts;i++) 
-      if(Hosts[i].host) 
+    for(i = 0;i < NHosts;i++)
+      if(Hosts[i].host)
 	free (Hosts[i].host);
     free(Hosts);
-  } 
+  }
 
   NHosts = 0;
 
@@ -108,14 +108,14 @@ ReadHosts(char *HostFile)
 	  NHosts++;
       }
     }
-    
-  if((Hosts = (struct wkstruct *)malloc(sizeof(struct wkstruct) * NHosts)) 
+
+  if((Hosts = (struct wkstruct *)malloc(sizeof(struct wkstruct) * NHosts))
      == NULL)
     {
       fprintf(stderr, "ERROR allocating memory for hosts struct\n");
       exit(2);
     }
-    
+
   if(fseek(QHosts, 0, SEEK_SET) != 0)
     {
       fprintf(stderr, "ERROR seeking file %s\n",HostFile);
@@ -126,7 +126,7 @@ ReadHosts(char *HostFile)
       Hosts[i].host = NULL;
       Hosts[i].load = 1e09;
     }
-    
+
   for(i = 0;i < NHosts;)
     {
       int j = 0, k;
@@ -143,9 +143,9 @@ ReadHosts(char *HostFile)
 	}
       }
       /*WGK: Truncate host at first white space character (e.g., newline.)*/
-       
-      for(k=j;k<strlen(TmpStr);k++) 
-	if(TmpStr[k] == ' ' | TmpStr[k] == '\t' | TmpStr[k] == '#' 
+
+      for(k=j;k<strlen(TmpStr);k++)
+	if(TmpStr[k] == ' ' | TmpStr[k] == '\t' | TmpStr[k] == '#'
 	   | TmpStr[k] == '\n' | TmpStr[k] == '\r') {
 	TmpStr[k] = '\0';
 	break;
@@ -161,18 +161,18 @@ ReadHosts(char *HostFile)
 	{
 	  /*WGK 1999/01/25 Bugfix. This is probably a bogus host.*/
           fprintf(stderr,
-		  "WARNING: Unable to resolve %s or out of memory\n", 
+		  "WARNING: Unable to resolve %s or out of memory\n",
 		  TmpStr[j]);
           NHosts--;
 	  continue;
         }
       i++;
-      
+
     }
   return(fclose(QHosts));
 }
 
-char 
+char
 *localhost(char *host)
 {
   /*Author: WGK*/
@@ -185,7 +185,7 @@ char
   return(lhost);
 }
 
-int 
+int
 countdots(char *host)
 {
   int count = 0;
@@ -230,7 +230,7 @@ canonicalhost(char *host)
   cache[MAXHOSTNAMELEN-1] = 0;
 
   /*Appending the domainname from getdomainname() as someone suggested
-    will not work because 
+    will not work because
     getdomainname is the NIS domain, not the BIND domain, and they are quite
     different in many installations! I've reluctantly given up and now
     simply return an unqualified domainname if we have a lazy gethostbyname().*/
@@ -247,7 +247,7 @@ canonicalhost(char *host)
 
   find_maxdots(cache, maxdots);
   find_maxdots(thathost->h_name, maxdots);
-  for(i = 0;thathost->h_aliases[i]!=NULL;i++) 
+  for(i = 0;thathost->h_aliases[i]!=NULL;i++)
     find_maxdots(thathost->h_aliases[i],maxdots);
 
   /*Convert host to lowercase.*/
@@ -298,7 +298,7 @@ readcookie(void)
 	| (sbuf.st_mode & S_IRGRP) | (sbuf.st_mode & S_IROTH))
       fprintf(stderr, "readcookie: %s: mode %03o allows general read and/or write; cookiefile must be accessible only by you.\n",
 	      COOKIEFILE, sbuf.st_mode), exit(2);
-    if(!(cookiefile=fopen(COOKIEFILE, "r")) 
+    if(!(cookiefile=fopen(COOKIEFILE, "r"))
        || !fgets(&cookie[0], 79, cookiefile)) {
       fprintf(stderr, "Unable to read valid cookiefile %s!\n", COOKIEFILE);
       exit(2);
@@ -348,3 +348,51 @@ unique(void)
     & 0x7fffffff;
 }
 
+
+
+/* netfwrite/netfread */
+/* These two routines should be used when writing/reading binary data
+ * to/from the job control files to ensure cross-platform operation.
+ *
+ * The calls are identical to the fwrite/fread functions (especially
+ * for big-endian systems :).
+ */
+size_t netfwrite (void *databuf, size_t size, size_t nmemb, FILE *stream) {
+
+  int i,j;
+  size_t rc,count;
+
+#ifdef BIG_ENDIAN_HOST
+  rc=fwrite(databuf, size, nmemb, stream);
+#else
+  count=0;
+  for(i=0;i<nmemb;i++) {
+    for (j=0;j<size;j++) {
+      count+=fwrite(databuf + (i*size) + (size-1-j), 1,1,stream);
+    }
+  }
+  rc = count / size;
+#endif
+
+  return(rc);
+}
+
+size_t netfread (void *databuf, size_t size, size_t nmemb, FILE *stream)
+{
+  int i,j;
+  size_t count,rc;
+
+#ifdef BIG_ENDIAN_HOST
+  rc=fread(databuf, size, nmemb, stream);
+#else
+  count=0;
+  for (i=0;i<nmemb;i++) {
+    for (j=0;j<size;j++) {
+      count += fread(databuf + (i*size) + (size-1-j), 1, 1, stream );
+    }
+  }
+  rc=count/size;
+#endif
+
+  return(rc);
+}
