@@ -1,10 +1,12 @@
 /*
  *  Queue load balancing system
- *  $Revision: 1.9 $
+ *  $Revision: 1.11 $
  *
- *  Copyright (C) 1998-2000 W. G. Krebs
+ *  Copyright (C) 1998-2000 W. G. Krebs <wkrebs-gnu.org> (except where otherwise noted.)
  *
  *  <wkrebs@gnu.org>
+ * 
+ *  This file is part of the GNU Queue package, http://www.gnuqueue.org .
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,19 +23,31 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *
- *  If you make modifications to the source, I would be happy to have
- *  them to include in future releases.  Feel free to send them to:
- *      W. G. Krebs	      				
- *	<wkrebs@gnu.org>
+ *  Please report bugs to <bug-queue@gnu.org>.
+ * 
  *
  **************************************************************************/
 
+
+
+
 #include "queue.h"
 
+/*WGK 2000/10/22 Switch to control the addition of the QUEUE_MANAGER package, submitted by Monica Lau <mllau@alantro.com>.*/
+
+#ifndef NO_QUEUE_MANAGER
+#define QUEUE_MANAGER 1
+#endif
+
+/* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+#ifdef QUEUE_MANAGER
+#include "queue_define.h"
+#endif /*QUEUE_MANAGER*/
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+
 #define SPOOLDIR QUEUEDIR
 
-#define sock_error(socket, format) write(socket, format, strlen(format));
+#define sock_error(socket, format) write(socket, format, strlen(format)); 
 
 extern char *getenv(const char *);
 /*extern char *index();*/
@@ -111,8 +125,8 @@ void error (char *s);
 void cleanup(void);
 
 /*WGK: The following is for do_queue part of the system.*/
-void
-send_signal(int signal)
+void 
+send_signal(int signal) 
 {
   char sig;
   /*Sig pipe means queued has died.*/
@@ -162,9 +176,51 @@ send_signal(int signal)
   }
 }
 
+/* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+/* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
+#ifdef QUEUE_MANAGER
+
+/* The # of licenses that we got on the command line. */
+int license_counter = 0;
+
+/* Structure used to store a license name. */
+struct license
+{
+	char name[MAXLICENSELEN];
+};
+
+#endif /*QUEUE_MANAGER*/
+/* End section copyrighted by Texas Instruments.*/
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+
 int
 main(int argc, char **argv, char **arge)
 {
+  /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+  /* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
+#ifdef QUEUE_MANAGER
+
+  /* packet to send over to the queue manager */
+  struct info_packet packet;
+
+  /* the current directory */
+  char cur_dir[MAXLOGFILE];
+
+  /* variable that specifies whether we have to connect to the queue manager or not
+   * (by default, we have to connect to it) */
+  int qmanager_connect = 1;
+
+  /* variables used within case 'a' for licenses */
+  int l_index;
+  struct license *l_array = NULL, *tl_array = NULL;
+
+  /* variable bit used to let us know if there is a log file or not
+   * (used within case 'e'); 0 means no file and 1 means there is a file */
+  int logfile = 0;
+
+  #endif /*QUEUE_MANAGER*/
+/* End section copyrighted by Texas Instruments.*/
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
 
   /*WGK: Some of these may be vestiges.*/
   register int i;
@@ -236,17 +292,69 @@ main(int argc, char **argv, char **arge)
 
   /* DVL : Open hosts file and read hosts */
 
+/* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+/* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
+#ifdef QUEUE_MANAGER
+
+/* set DISPLAY if "DISPLAY=:0" */
+{
+	char display_cmd[50] = "DISPLAY=";
+	char hostname[30];
+	char display[30];
+	char *dgetenv;
+	int set_display = 0;
+
+	strcpy(hostname, getenv("HOSTNAME"));
+	dgetenv = getenv("DISPLAY");
+
+	/* if DISPLAY is not blank */
+	if ( dgetenv != NULL )
+	{
+	   strcpy(display, dgetenv);
+	   if ( display[0] == ':' )  /* if DISPLAY is something like :0 */
+	   {
+	      strcat(display_cmd, hostname);
+	      strcat(display_cmd, display);
+	      strcat(display_cmd, ".0");
+	      set_display = 1;
+	   }
+	}
+
+	/* need to set display? */
+	if ( set_display )
+		putenv(display_cmd);
+}
+
+#endif /*QUEUE_MANAGER*/
+  /* End of code copyrighted by Texas Instruments.*/
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
 
   /*
    * Parse command line.
    */
 
-  optionstring = "h:H:d:opnqj:iwrv";
+  /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+#ifdef QUEUE_MANAGER
+  optionstring = "h:H:d:opnqj:iwrva:ce:1";
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+#else
+ optionstring = "h:H:d:opnqj:iwrv";
+#endif /*QUEUE_MANAGER*/
+
   if(invoke = rindex(argv[0], '/')) invoke++;
   else invoke = argv[0];
 
   if(!strcmp(invoke, "qsh")) {
+
+#ifdef QUEUE_MANAGER
+    /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+    optionstring = "+l:d:opnj:iwrva:ce:1";
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+#else
     optionstring = "+l:d:opnj:iwrv";
+#endif /*QUEUE_MANAGER*/
+
+
     /*rsh compatibility mode defaults.*/
     ttyoutput = 0;
     ttyinput = 0;
@@ -254,6 +362,85 @@ main(int argc, char **argv, char **arge)
     donthang = 0;
   }
 
+/* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+/* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
+#ifdef QUEUE_MANAGER
+
+/* initialize the packet structure to some default values */
+strcpy(packet.mode, "interactive");
+strcpy(packet.priority, "low");
+strcpy(packet.prefhost, "");
+strcpy(packet.onlyhost, "");
+
+/* get the user name */
+{
+   char *user = getenv("USER");
+   strcpy(packet.user, user);
+}
+
+/* get the real user id */
+packet.user_id = getuid();
+
+/* get the real user group id */
+packet.gid = getgid();
+
+/* get the date */
+{
+   int i, n, pid, pfd[2];
+   char buffer[30];
+
+   if ( pipe(pfd) == -1 )
+   {
+	   fprintf(stderr, "Queue.c Error: can't pipe() to get the date\n");
+	   return -1;
+   }
+
+   if ( (pid = fork()) == 0 )
+   {
+	   /* child process */
+	   close(pfd[0]);
+	   close(1);
+	   dup(pfd[1]);
+	   execlp("date", "date", NULL);
+   }
+   else if ( pid == -1 )
+   {
+	   fprintf(stderr, "Queue.c Error: can't fork() off process to get the date\n");
+	   return -1;
+   }
+
+   /* parent process */
+   close(pfd[1]);
+
+   /* read from the pipe */
+   if ( (n = read(pfd[0], buffer, sizeof(buffer))) < 0 )
+   {
+	   fprintf(stderr, "Queue.c Error: can't read from pipe to get the date\n");
+	   return -1;
+   }
+   close(pfd[0]);
+
+   /* wait to bury child process */
+   wait(NULL);
+
+   buffer[n] = '\0';
+
+   /* remove the newline character from buffer */
+   for (i = 0; i < sizeof(buffer); i++)
+      if (buffer[i] == '\n')
+	   buffer[i] = '\0';
+
+   strcpy(packet.datesubmit, buffer);
+}
+
+/* get the current working directory */
+getcwd(cur_dir, MAXLOGFILE);
+strcpy(packet.cur_dir, cur_dir);
+
+#endif /*QUEUE_MANAGER*/
+/*End of code copyrighted by Texas Instruments.*/
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+
   while ((i = getopt_long(argc, argv, optionstring, longopts, NULL)) != EOF)
 
     switch (i) {
@@ -268,10 +455,31 @@ main(int argc, char **argv, char **arge)
 	fprintf(stderr, "Queue: hostname too long.\n");
 	exit(1);
       }
+
+      /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+      /* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
+#ifdef QUEUE_MANAGER
+      strcpy(packet.prefhost, optarg);
+      strcpy(packet.onlyhost, optarg);
+
+      /* lowercase the host names */
+      {
+      int j;
+      for (j = 0; j < strlen(packet.prefhost); j++)
+	      packet.prefhost[j] = tolower(packet.prefhost[j]);
+
+      for (j = 0; j < strlen(packet.onlyhost); j++)
+	      packet.onlyhost[j] = tolower(packet.onlyhost[j]);
+      }
+
+      #endif /*QUEUE_MANAGER*/
+      /*End of code copyrighted by Texas Instruments.*/
+      /* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+
       prefhost = strdup(canonicalhost(optarg));
       if(!validhost(prefhost, 0)) {
-	fprintf(stderr,
-		"Queue: invalid queue hostname: %s\n", prefhost);
+	fprintf(stderr, 
+		"Queue: invalid queue hostname: %s\n", prefhost); 
 	exit(1);
       }
       onlyhost = prefhost;
@@ -282,9 +490,26 @@ main(int argc, char **argv, char **arge)
 	fprintf(stderr, "Queue: hostname too long.\n");
 	exit(1);
       }
+
+      /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+      /* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
+#ifdef QUEUE_MANAGER
+      strcpy(packet.prefhost, optarg);
+
+      /* lowercase the host name */
+      {
+      int j;
+      for (j = 0; j < strlen(packet.prefhost); j++)
+	      packet.prefhost[j] = tolower(packet.prefhost[j]);
+      }
+
+      #endif /*QUEUE_MANAGER*/
+      /*End of code copyrighted by Texas Instruments.*/
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+
       prefhost = strdup(canonicalhost(optarg));
       if(!validhost(prefhost, 0)) {
-	fprintf(stderr, "Queue: invalid queue hostname: %s\n", prefhost);
+	fprintf(stderr, "Queue: invalid queue hostname: %s\n", prefhost); 
 	exit(1);
       }
       prefhost=prefhost;
@@ -330,8 +555,17 @@ main(int argc, char **argv, char **arge)
       donthang = 0;
       break;
     }
-    case 'r': {
+    case 'r': {  /* user requests batch mode */
+      /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+#ifdef QUEUE_MANAGER
+      strcpy(packet.mode, "batch");  
+#else
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+      /*2000/10/22 WGK allow donthang to be set properly if QUEUE_MANAGER is compiled out.*/
       donthang = 1;
+#endif /*QUEUE_MANAGER*/
+
+
       break;
     }
     case 'v':
@@ -341,12 +575,369 @@ main(int argc, char **argv, char **arge)
 	fprintf(stderr, "Bug reports to bug-queue@gnu.org.\n");
 	exit(0);
       }
-		
+      /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+      /* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
+#ifdef QUEUE_MANAGER
+case 'a':  /* to get the license names */
+{
+/* increment the number of licenses */
+license_counter++;
+
+/* if l_array is not NULL, need to deallocate the memory */
+if ( l_array != NULL )
+	free( (void *) l_array );
+
+/* dynamically allocate l_array to hold all the license names */
+l_array = (struct license *)calloc(license_counter, sizeof(struct license));
+
+/* copy the contents of tl_array to l_array (if license_counter != 1),
+ * along with the new license name */
+if ( license_counter != 1 )
+{
+   for (l_index = 0; l_index < license_counter - 1; l_index++)
+	   l_array[l_index] = tl_array[l_index];
+}
+
+for(l_index = 0; l_index < strlen(optarg); l_index++)
+	l_array[license_counter - 1].name[l_index] = optarg[l_index];
+
+l_array[license_counter - 1].name[l_index] = '\0';
+
+/* now dynamically allocate tl_array, copy everything from l_array to
+ * tl_array */
+tl_array = (struct license *)calloc(license_counter, sizeof(struct license));
+for (l_index = 0; l_index < license_counter; l_index++)
+	tl_array[l_index] = l_array[l_index];
+
+break;
+}
+case 'c':  /* user requests high priority */
+{
+	strcpy(packet.priority, "high");
+	break;
+}
+case 'e':  /* user specifies log file (only used if in batch mode) */
+{
+	logfile = 1;
+	strcpy(packet.logfile, optarg);
+	break;
+}
+case '1':  /* this is a hack, so that we can specify not to go to the
+	      queue manager; this hack is probably not such a good idea because
+	      ordinary users can discover this option, but it will have to do for now */
+{
+	qmanager_connect = 0;
+	break;
+}
+#endif /*QUEUE_MANAGER*/
+/* End of code copyrighted by Texas Instruments.*/
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+
     default:
-      if(strcmp(invoke, "qsh")==0) Qshusage();
+      if(strcmp(invoke, "qsh")==0) Qshusage(); 
       Usage();
     }
 
+/* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+/* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
+#ifdef QUEUE_MANAGER
+
+/* get the job command */
+{
+   int i;
+   strcpy(packet.job, argv[optind]);
+   for(i = optind + 1; i < argc; i++)
+	   sprintf(packet.job, "%s %s", packet.job, argv[i]);
+
+   /* parse the command to see if there are any '|'s in there ('|'s do not work in Queue software) */
+   for (i = 0; i < sizeof(packet.job); i++)
+      if (packet.job[i] == '|')
+      {
+	 fprintf(stderr, "Queue.c Error: no |'s allowed in Queue software\n");
+	 return -1;
+      }
+}
+
+/* if tl_array is not NULL, need to deallocate its memory */
+if ( tl_array != NULL )
+	free( (void *) tl_array );
+
+/* check to make sure that user specified license(s) */
+if ( license_counter == 0 )
+{
+	fprintf(stderr, "Queue.c Error: must specify license(s)\n");
+	return -1;
+}
+else
+{
+	/* lowercase all the license(s) */
+	int i, j;
+	for (i = 0; i < license_counter; i++)
+	{
+	   for (j = 0; j < strlen(l_array[i].name); j++)
+		   (l_array[i].name)[j] = tolower( (l_array[i].name)[j] );
+	}
+}
+
+/* if user specified batch mode, check to make sure that we have a log file */
+if ( !strcasecmp(packet.mode, "batch") )
+{
+   if ( logfile != 1 )  /* means, there's no log file */
+   {
+	   fprintf(stderr, "Queue.c Error: in batch mode, must specify log file\n");
+	   return -1;
+   }
+   else  /* determine the full path of the log file */
+   {
+      /* if user did not specify an absolute path or if user did not specify
+       * tilde (shell automically replaces '~' with the home directory) */
+      if (packet.logfile[0] != '/')
+      {
+	 /* if the user specified "./", then need to get the current directory */
+	 if ( packet.logfile[0] == '.' )
+	 {
+	    int i, j;
+	    char logfile[MAXLOGFILE];
+
+	    /* delete '.' from the logfile */
+	    for ( i = 0, j = 1; j < strlen(packet.logfile); i++, j++ )
+		    logfile[i] = packet.logfile[j];
+	    logfile[i] = '\0';
+
+	    /* concatenate the current directory with the logfile */
+	    strcat(cur_dir, logfile);
+	    strcpy(packet.logfile, cur_dir);
+	 }
+	 /* else, the user specified a relative path, need to get current directory */
+	 else
+	 {
+	    /* concatenate the current directory with the logfile */
+	    strcat(cur_dir, "/");
+	    strcat(cur_dir, packet.logfile);
+	    strcpy(packet.logfile, cur_dir);
+	 }
+      }
+
+      /* MY_DEBUG */
+      printf("LOGFILE: %s\n", packet.logfile);
+   }
+}
+
+/* if user specified interactive mode, make sure that job is high priority */ 
+if ( !strcasecmp(packet.mode, "interactive") )
+	strcpy(packet.priority, "high");
+
+/* Now make a connection to the queue_manager and send all the information to it. */
+if ( qmanager_connect )
+{
+   char buffer[MAXHOSTNAMELEN];
+   /* this is the assigned host that we get back from the queue manager */
+   char assigned_host[MAXHOSTNAMELEN];
+
+   /* socket variables */
+   int sockfd;
+   struct sockaddr_in serv_addr;
+   struct hostent *host;
+   int sendbuff = 16384;
+
+   /* counter to keep track of the number of times we try connecting to the q_manager */
+   int counter = 1;
+
+   strcpy(assigned_host, ""); /* initialize assigned_host */
+   strcpy(buffer, ""); /* initialize buffer */
+
+   /* Get the IP address of the host name */
+   if ( (host = gethostbyname(QMANAGERHOST)) == NULL )
+   {
+	   fprintf(stderr, "Queue.c Error: invalid host name of the queue manager\n");
+	   return -1;
+   }
+
+   /* Fill in the server information so that we can connect to it */
+   serv_addr.sin_family = AF_INET;
+   serv_addr.sin_port = htons(PORTNUM2);
+   serv_addr.sin_addr = *((struct in_addr *)host->h_addr);
+
+   /* If in interactive mode, we will try to connect ten times to prevent an
+    * overloaded system from  keeping us down.  If in batch mode, we will try
+    * connecting forever until a connection is successful. */
+
+   for(;;)
+   {
+      /* Open a TCP socket */
+      if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
+      {
+	      fprintf(stderr, "Queue.c Error: can't get TCP socket\n");
+	      return -1;
+      }
+
+      /* Set socket options for TCP socket */
+      if ( setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *) &sendbuff,
+			      sizeof(sendbuff)) < 0 )
+      {
+	      fprintf(stderr, "Queue.c Error: can't set TCP setsock options\n");
+	      return -1;
+      }
+
+      if ( connect(sockfd, (struct sockaddr *) &serv_addr,
+			      sizeof(serv_addr)) < 0 )
+      {
+	      printf("Connecting to Queue_Manager...\n");
+	      sleep(3); /* sleep a few seconds before trying to connect again */
+	      close(sockfd);  /* close off the socket */
+
+	      /* interactive mode */
+	      if ( !strcmp(packet.mode, "interactive") )
+	      {
+		      ++counter;
+
+		      /* Is queue manager down? */
+		      if ( counter == 11 )
+		      {
+			      fprintf(stderr, "Queue.c Error: Queue_Manager is either down or is very busy.  Try again later.\n");
+			      return -1;
+		      }
+	      }
+      }
+      else
+	break;
+   }
+
+   /* First, send the packet structure. */
+   if ( send(sockfd, &packet, sizeof(packet), 0) < 0 )
+   {
+	   fprintf(stderr, "Queue.c Error: can't send packet to queue manager\n");
+	   return -1;
+   }
+
+   /* Next, send the license(s). */
+   for (l_index = 0; l_index < license_counter; l_index++)
+   {
+	   if ( send(sockfd, l_array[l_index].name, MAXLICENSELEN, 0) < 0 )
+	   {
+		   fprintf(stderr, "Queue.c Error: can't send license name(s) to queue manager\n");
+		   return -1;
+	   }
+   }
+
+   /* Send an "EOF" message to let queue manager know that this is the end
+    * of the transaction. */
+   strcpy(l_array[0].name, "EOF");
+   if ( send(sockfd, l_array[0].name, MAXLICENSELEN, 0) < 0 )
+   {
+	   fprintf(stderr, "Queue.c Error: can't send EOF message to queue_manager\n");
+	   return -1;
+   }
+
+   /* Finally, if we're in batch mode, need to send the environment to the queue_manager. */
+   if ( !strcasecmp(packet.mode, "batch") )
+   {
+	   /* send string to the queue_manager one at a time (most simplistic method) */
+	   char data[MAXSTRINGLEN];
+	   struct timeval timeout;
+	   char ** arge_index = arge;
+
+	   while ( *arge_index != 0 )
+	   {
+	      strcpy(data, *arge_index);
+	      if ( send(sockfd, data, MAXSTRINGLEN, 0) < 0 )
+	      {
+		      fprintf(stderr, "Queue.c Error: can't send environment to queue_manager\n");
+		      return -1;
+	      }
+
+	      /* sleep for 50 microseconds before sending the next string */
+	      timeout.tv_sec = 0;
+	      timeout.tv_usec = 50;
+	      if ( select(0, NULL, NULL, NULL, &timeout) < 0 )
+		      fprintf(stderr, "Queue.c Error: can't set timer to send the environment string\n");
+
+	      ++arge_index;
+	   }
+
+	   /* Send an "EOF" message to let queue manager know that this is the end
+	    * of the transaction. */
+	   strcpy(data, "EOF");
+	   if ( send(sockfd, data, MAXSTRINGLEN, 0) < 0 )
+	   {
+		   fprintf(stderr, "Queue.c Error: can't send EOF message to queue_manager\n");
+		   return -1;
+	   }
+
+	   /* MY_DEBUG */
+	   printf("After sending environment to qmanager\n");
+   }
+
+   /* if in batch mode, just wait for an ack (job_id) from queue_manager and exit */
+   if ( !strcasecmp(packet.mode, "batch") )
+   {
+	   if ( recv(sockfd, buffer, sizeof(buffer), 0) < 0 )
+	   {
+		   fprintf(stderr, "Queue.c Error: can't receive ack from queue_manager\n");
+		   return -1;
+	   }
+
+	   /* did we get an error message? */
+	   if ( !strcmp(buffer, "-1") )
+	   {
+		   fprintf(stderr, "Queue.c Error: user entered either invalid licenses or hosts\n");
+		   return -1;
+	   }
+	   else  /* print out the job_id */
+		   printf("JOB_ID: %s\n", buffer);
+
+	   return 0;
+   }
+
+   /* now wait for an assigned host */
+   if ( recv(sockfd, assigned_host, sizeof(assigned_host), 0) < 0 )
+   {
+	   fprintf(stderr, "Queue.c Error: can't receive assigned host name\n");
+	   return -1;
+   }
+
+   /* did we get an error message? */
+   if ( !strcmp(assigned_host, "-1") )
+   {
+	   fprintf(stderr, "Queue.c Error: user entered either invalid licenses or hosts\n");
+	   return -1;
+   }
+
+   /* wait for the job id */
+   if ( recv(sockfd, buffer, sizeof(buffer), 0) < 0 )
+   {
+	   fprintf(stderr, "Queue.c Error: can't receive job id\n");
+	   return -1;
+   }
+
+   /* close off the connection */
+   close(sockfd);
+
+   /* now assign prefhost and onlyhost to this assigned host */
+   prefhost = strdup(canonicalhost(assigned_host));
+   onlyhost = prefhost;
+
+   /* MY_DEBUG */
+   printf("ASSIGNED_HOST: %s\n", onlyhost);
+   printf("JOB_ID: %s\n", buffer);
+
+   /* if we didn't get an assigned host, we should exit */
+   if ( !strcmp(assigned_host, "") )
+   {
+	   fprintf(stderr, "Queue.c Error: did not get an assigned host\n");
+	   return -1;
+   }
+
+} /* end of block */
+
+/* if l_array is not NULL, need to deallocate its memory */
+if ( l_array != NULL )
+	free( (void *) l_array );
+
+#endif /*QUEUE_MANAGER*/
+/* End of code copyrighted by Texas Instruments.*/
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+
   /*WGK 98/12/24 If we're running 'qsh', next thing is the hostname,
     not the command line. This is then followed by the command line;
     no '--' is used with qsh.*/
@@ -508,7 +1099,7 @@ main(int argc, char **argv, char **arge)
 	  }
 	  perror("queue: bind");
 	  exit(2);
-	}
+	}               
 
       }
     }
@@ -616,14 +1207,15 @@ main(int argc, char **argv, char **arge)
     if(!(myhost = gethostbyname(buf))) {
       /*WGK 98/12/19 prevent seg fault if DNS goes down.*/
       setuid(getuid()); /*Drop privileges*/
-      fprintf(stderr,
+      fprintf(stderr, 
 	      "queue: gethostbyname failed; can't get my IP address.\n");
       herror("queue");
       fprintf(stderr, "queue: DNS is probably down and resolv+ isn't configured to get the IP address from /etc/hosts.\n");
       exit(2);
     }
+
     /* EJD -- 8/14/2000 -- host address is already network order - Don't swap */
-    fwrite(myhost->h_addr, sizeof(address1.sin_addr), 1, queue);
+     fwrite(myhost->h_addr, sizeof(address1.sin_addr), 1, queue);
     netfwrite(&miport, sizeof(int), 1, queue);
 
     {int temp;
@@ -771,7 +1363,7 @@ main(int argc, char **argv, char **arge)
 	prot = IPPROTO_IP;
 
       optionsize = sizeof(optionbuf);
-      if (getsockopt(newsocket, prot, IP_OPTIONS,
+      if (getsockopt(newsocket, prot, IP_OPTIONS, 
 		     (char *) optionbuf, &optionsize) == 0
 	  && optionsize != 0) {
 	/* We probably don't need this since we're checking to make
@@ -823,7 +1415,7 @@ Have your administrator fix identd problems on your cluster, OR recompile withou
       exit(2);
     }
 #endif /*HAVE_IDENTD*/
-#endif /*NO_ROOT*/
+#endif /*NO_ROOT*/ 
 
 	 /*#if defined(HAVE_IDENTD) || !defined(NO_ROOT) || defined(NO_SECURITY)*/
 	 /*WGK 1998/12/25 get speed back if crypt() isn't needed because portsare privileged or verifiable.*/
@@ -869,7 +1461,7 @@ DFPRINTF3(stderr,"cnounce %s\n",cnounce);
     randomstr(&nounce[0]);
     DFPRINTF3(stderr,"nounce %s\n",nounce);
     write(newsocket, &nounce, strlen(nounce)+1);
-
+    
     strcpy(bigcookie,nounce);
     strcat(bigcookie,cookie);
     strcat(bigcookie,cnounce);
@@ -905,7 +1497,7 @@ DFPRINTF3(stderr,"cnounce %s\n",cnounce);
       write(newsocket, &cnounce, strlen(cnounce)+1);
       DFPRINTF3(stderr,"wrote cnounce %s\n",cnounce);
       /*Read nounce, as compliant with draft protocol. WGK 2000/07/20*/
-
+ 
       alarm(5);
     k = 0;
     do {
@@ -928,7 +1520,7 @@ DFPRINTF3(stderr,"cnounce %s\n",cnounce);
 flag = 0;
       write(newsocket, &randomcookie[0], strlen(randomcookie)+1);
        /*Wait for the OK byte from other side indicating authentication has been successful.*/
-
+  
       alarm(10);
       read(newsocket, &c, 1);
       alarm(0);
@@ -996,7 +1588,7 @@ flag = 0;
       perror("queue: problem getting second socket");
       exit(1);
     }
-
+ 
 #endif /*NO_ROOT*/
 
     address1.sin_port = htons((u_short) secondport);
@@ -1024,7 +1616,7 @@ flag = 0;
       for ( ; ; hostp->h_addr_list++) {
 	if (bcmp(hostp->h_addr_list[0], (caddr_t) &address1.sin_addr, sizeof(address1.sin_addr)) == 0)
 	  break; /* reverse lookup test passed.*/
-	
+	    
 	if (hostp->h_addr_list[0] == NULL) {
 	  fprintf(stderr,
 		  "Host addr %s not listed for host %s", inet_ntoa(address1.sin_addr), hostp->h_name);
@@ -1036,9 +1628,9 @@ flag = 0;
       strncpy(remotehost, hostp->h_name, sizeof(remotehost) - 1);
       remotehost[sizeof(remotehost) - 1] = 0;
       hostname = canonicalhost(remotehost);
-    }
+    } 
     else
-      hostname = "unresolved";
+      hostname = "unresolved"; 
     /*Our DNS must be down and this will probably fail.*/
 
     if(!validhost(hostname, newsocket)) exit(1);
@@ -1085,7 +1677,7 @@ flag = 0;
     fdsocket3 = dup(fdsocket2);
 
     /*WGK: OOB messages will contain information about wait signals
-      from the remote job.
+      from the remote job.  
       Non-OOB is stderr.*/
 
     setsockopt(fdsocket3, SOL_SOCKET, SO_OOBINLINE, &one, sizeof(one));
@@ -1126,7 +1718,7 @@ flag = 0;
       /*Block all other signals so that signals are delivered in order
         received.*/
       mysignal.sv_mask= ~0L;
-      for(i=0;i<64;++i) sigvector(i, &mysignal, NULL);
+      for(i=0;i<64;++i) sigvector(i, &mysignal, NULL);    
 #else  /*GNU/Linux*/
       struct sigaction mysignal;
       mysignal.sa_handler = send_signal;
@@ -1152,10 +1744,10 @@ flag = 0;
 	uses the other 64 signals that are partially defined in
 	<sigset.h>*/
       mysignal.sa_flags = SA_RESTART;
-      for(i=0;i<64;++i) sigaction(i, &mysignal, NULL);
+      for(i=0;i<64;++i) sigaction(i, &mysignal, NULL);    
 #endif
       sigsetmask(0L);
-    }
+    }     
 
 #ifdef ENABLE_CHECKPOINT
     if(checkpointed) {
@@ -1189,7 +1781,7 @@ flag = 0;
 	  this read get interrupted by signals at all costs!.*/
 	long oldmask;
 	oldmask=sigsetmask(~0L);
-	if ((cint=read(newsocket, buf, sizeof(buf))) <= 0) {
+	if ((cint=read(newsocket, buf, sizeof(buf))) <= 0) { 
 
 
 	  /*  if (errno!= EINTR) FD_CLR(newsocket, &readf);*/
@@ -1285,11 +1877,11 @@ flag = 0;
 
 	      /*Again, this is thought to be the most portable
 		version:*/
-
+  
 	      *((unsigned long int*) &mysignal.sa_mask) = ~0L;
 
 
-
+  
 	      mysignal.sa_flags = SA_RESTART;
 #endif
 
@@ -1343,7 +1935,7 @@ flag = 0;
 	/* Turn off OOB inline mode so as not to read it in now.*/
 	setsockopt(fdsocket3, SOL_SOCKET, SO_OOBINLINE, &zero, sizeof(one));
 	if ((cint=read(fdsocket2, buf, sizeof(buf))) >0 ){
-	  write(2, buf, cint);
+	  write(2, buf, cint);  
 	}
 	else {FD_CLR(fdsocket2, &readf);}
 	/* Turn OOB inline back on so that select() works.*/
@@ -1380,7 +1972,7 @@ cleanup(void)
 
 /*Read a string from the socket making sure that it fits into our buffer.*/
 void
-getstr(int socket, char *buffer,
+getstr(int socket, char *buffer, 
        int size, /* size of the char array */
        char *errm /* what to say if string is too long.*/)
 {
@@ -1404,7 +1996,7 @@ Qshusage(void)
 {
 
   fflush(stdout);
-  fprintf(stderr, "Queue load balancing system \t http://www.gnuqueue.org .\n\nCopyright 1998-2000 W. G. Krebs <wkrebs@gnu.org>.\n\nPlease report bugs to <bug-queue@gnu.org>.\n\nThere is absolutely no warranty for queue; see file COPYING for details.\n\n");
+  fprintf(stderr, "Queue load balancing system \t http://www.gnuqueue.org .\n\nCopyright 1998-2000 W. G. Krebs <wkrebs@gnu.org> and others.\n\nPlease report bugs to <bug-queue@gnu.org>.\n\nThere is absolutely no warranty for queue; see file COPYING for details.\n\n");
   fprintf(stderr, "qsh is Queue's rsh compatibility mode.\n\n");
   fprintf(stderr, "Usage:\n  qsh [-l ignored] hostname command command.options\n\n-l option is ignored for rsh compatbility.\n\nIf hostname is set to '+', hostname is left up to GNU Queue.\n");
   exit(1);
@@ -1415,15 +2007,29 @@ Usage(void)
 {
 
   fflush(stdout);
-  fprintf(stderr, "Queue load balancing system \t http://www.gnuqueue.org .\n\nCopyright 1998-2000 W. G. Krebs <wkrebs@gnu.org>.\n\nPlease report bugs to <bug-queue@gnu.org>.\n\nThere is absolutely no warranty for queue; see file COPYING for details.\n\n");
+  fprintf(stderr, "Queue load balancing system \t http://www.gnuqueue.org .\n\nCopyright 1998-2000 W. G. Krebs <wkrebs@gnu.org> and others.\n\nPlease report bugs to <bug-queue@gnu.org>.\n\nThere is absolutely no warranty for queue; see file COPYING for details.\n\n");
   fprintf(stderr, "Usage:\n   queue [-h hostname] [-i|-q] [-o|-p|-n] [-w|-r] [-j jobname] [-d spooldir] -- command command.options\n");
   fprintf(stderr, "-h hostname or --host hostname: force queue to run on hostname.\n");
   fprintf(stderr, "-H hostname or --robust-host hostname: Run job on hostname if it is up.\n");
   fprintf(stderr, "-i|-q or --immediate|--queue: Toggle between immediate and queue mode, default is to queue, which requires specification of a spooldir.\n");
   fprintf(stderr, "[-d spooldir] or [--spooldir spooldir] specifies the name of the batch processing directory, e.g., gnulab\n");
   fprintf(stderr, "-o|-p|-n or --half-pty|--full-pty|--no-pty: Toggle between half pty emulation, full pty emulation, and the more efficient no pty emulation.\n");
-  fprintf(stderr, "-w|-r or --wait|--batch Toggle between wait (stub daemon) and return (mail batch) mode, default depends on invocation command.\n");
+#ifdef QUEUE_MANAGER
+  fprintf(stderr, "-w or --wait Wait (stub daemon).\n");
+#else
+ fprintf(stderr, "-w|-r or --wait|--batch Toggle between wait (stub daemon) and return (mail batch) mode, default depends on invocation command.\n");
+#endif /*QUEUE_MANAGER/*
   fprintf(stderr, "-v or --version: Version\n");
+
+  /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
+#ifdef QUEUE_MANAGER
+  fprintf(stderr,"-a license: Specifies a license that job needs in order to run.  If need to specify more than once license, then type in multiple '-a license'.\n");
+  fprintf(stderr, "-r: Specifies batch mode; default is interactive mode.\n");
+  fprintf(stderr, "-c: Specifies high priority job; default is low priority.\n");
+  fprintf(stderr, "-e logfile: Specifies the file name of a log file.\n");
+  #endif /*QUEUE_MANAGER*/
+/* 2000/10/22 End of code added by Monica Lau <mllau@alantro.com>*/
+
   fprintf(stderr, "--help: This menu\n");
   fprintf(stderr, "See documentation for full explanation of options.\n");
   fprintf(stderr, "\nExamples:\n to run interactive matlab remotely: queue -i -w -p -- matlab\n To get the best host to run jobs on: queue -i -w -- hostname\n");
@@ -1504,7 +2110,7 @@ syserror(char *s)
     syslog(LOG_ALERT, "queue: %s", s);
 }
 
-int
+int 
 validhost(char *hostcpy, int socket)
 {
   char buf[MAXHOSTNAMELEN + 1];
@@ -1512,7 +2118,7 @@ validhost(char *hostcpy, int socket)
   int flag = 1;
   char *hostname = buf;
 
-  hostname = strncpy(hostname, hostcpy, MAXHOSTNAMELEN);
+  hostname = strncpy(hostname, hostcpy, MAXHOSTNAMELEN); 
   /*Unfortunately, the static area from gethostbynames is completely
     unstable on some platforms and will not survive a call to stat(),
     which is used in readhosts.*/
@@ -1533,8 +2139,8 @@ validhost(char *hostcpy, int socket)
     fprintf(stderr, "Authorized hosts in %s are:\n", QHOSTSFILE);
     for(i=0;i<NHosts;++i) fprintf(stderr, "%s\n", Hosts[i].host);
     fprintf(stderr, "\n");
-    if(socket)
-      sock_error(socket,
+    if(socket) 
+      sock_error(socket, 
 		 "This server is not in QHOSTSFILE as a valid server host.\n");
     return(0);
   }
