@@ -1,9 +1,9 @@
 /*
  *  Queue load balancing system
- *  $Revision: 1.6 $
+ *  $Revision: 1.8 $
  *
  *  Copyright (C) 1998-2000 W. G. Krebs
- *  
+ *
  *
  *  wkrebs@gnu.org
  *
@@ -44,15 +44,15 @@
 
 extern char cookie[];
 
-void 
-donothing(void) 
+void
+donothing(void)
 {
   /*Signal handler.*/
 }
 
 
-int 
-compar_wk(struct wkstruct *one, struct wkstruct *two) 
+int
+compar_wk(struct wkstruct *one, struct wkstruct *two)
 {
   if (one->load == two->load) return(0);
   return( (one->load < two->load ? -1 : 1));
@@ -61,7 +61,7 @@ compar_wk(struct wkstruct *one, struct wkstruct *two)
 /*Wake up daemon if sleeping*/
 int w_fd, miport;
 
-int 
+int
 con_daemon(char *hostname, char *queuename, int migrate)
 {
   int count=0;
@@ -125,7 +125,7 @@ version of the job ontrol file (VERSION1 is the only version supported.*/
 Use half of sha1() mechanism used later on to confirm cookie without
     actually sending over the network. This will bring us into complaince
 with the draft protocol. Other means of authentication can be imagined,
-such as those based on a signature, which is essentially an 
+such as those based on a signature, which is essentially an
 assymetric authentication. At present, there is no compelling reason
 to favor assymetric authentication over symmetric authentication. If
 we reach the point where we have huge networks, we might want to
@@ -164,7 +164,7 @@ DFPRINTF3(stderr,"cnounce %s\n",cnounce);
     randomstr(&nounce[0]);
     DFPRINTF3(stderr,"nounce %s\n",nounce);
     write(w_fd, &nounce, strlen(nounce)+1);
-    
+
     strcpy(bigcookie,nounce);
     strcat(bigcookie,cookie);
     strcat(bigcookie,cnounce);
@@ -202,7 +202,7 @@ DFPRINTF3(stderr,"cnounce %s\n",cnounce);
       write(w_fd, &cnounce, strlen(cnounce)+1);
       DFPRINTF3(stderr,"wrote cnounce %s\n",cnounce);
       /*Read nounce, as compliant with draft protocol. WGK 2000/07/20*/
- 
+
       alarm(5);
     k = 0;
     do {
@@ -225,7 +225,7 @@ DFPRINTF3(stderr,"cnounce %s\n",cnounce);
 flag = 0;
       write(w_fd, &randomcookie[0], strlen(randomcookie)+1);
        /*Wait for the OK byte from other side indicating authentication has been successful.*/
-  
+
       alarm(10);
       read(w_fd, &c, 1);
       alarm(0);
@@ -251,7 +251,7 @@ flag = 0;
   return(w_fd);
 }
 
-float 
+float
 getrldavg(char *hostname, char *queuename, char migrate)
 {
   int count=0;
@@ -320,7 +320,9 @@ version of the job ontrol file (VERSION1 is the only version supported.*/
   /*SIGALRM will interrupt the fread.*/
   signal(SIGALRM, donothing);
   alarm(5);
-  fread(&load, sizeof(load), 1, argh);
+
+  netfread(&load, sizeof(load), 1, argh);
+
   alarm(0);
   fclose(argh);
   close(w_fd); /*Make sure it is closed.*/
@@ -332,7 +334,7 @@ version of the job ontrol file (VERSION1 is the only version supported.*/
   return(load);
 }
 
-int 
+int
 wakeup(char *queuename, int migrate, int contact, char *nothost)
 {
   int ret;
@@ -352,7 +354,7 @@ wakeup(char *queuename, int migrate, int contact, char *nothost)
     average is above loadsched, so not needed.*/
 
   ret= -1;
-  if(contact) 
+  if(contact)
     for(i=0;ret==-1 & i<NHosts; ++i ) {
 
       /*WGK 2000/07/30. Prevent from sending jobs to ourself, which is
